/*
 * arch/arm/mach-ep93xx/include/mach/ts72xx.h
 */

/*
 * TS72xx memory map:
 *
 * virt		phys		size
 * febff000	22000000	4K	model number register
 * febfe000	22400000	4K	options register
 * febfd000	22800000	4K	options register #2 (JP6 and TS-9420 flags)
 * febfc000	[67]0000000	4K	NAND data register
 * febfb000	[67]0400000	4K	NAND control register
 * febfa000	[67]0800000	4K	NAND busy register
 * febf9000	10800000	4K	TS-5620 RTC index register
 * febf8000	11700000	4K	TS-5620 RTC data register
 * febf7000	23800000	4K	CPLD watchdog (control register)
 * febf6000	23c00000	4K	CPLD watchdog (feed register)
 * febf5000	23400000	4K	PLD version (3 bits)
 * febf4000	22c00000	4K	RS-485 control register
 * febf3000	23000000	4K	RS-485 mode register
 * febf2000	10800000	4K	jumpers/max197 busy bit/COM1 dcd register (8-bit, read only)
 * febf1000	10f00000	4K	max197 sample/control register (16-bit read/8-bit write)
 * febf0000	11e00000	4K	PC/104 8-bit I/O
 * febef000	21e00000	4K	PC/104 16-bit I/O
 * fea00000	11a00000	1MB	PC/104 8-bit memory
 * fe900000	21a00000	1MB	PC/104 16-bit memory
 * xxxxxxxx     80840000        4k      TS-7400 RTC GPIO pins
 */

#define TS72XX_MODEL_PHYS_BASE		0x22000000
#define TS72XX_MODEL_VIRT_BASE		0xfebff000
#define TS72XX_MODEL_SIZE		0x00001000

#define TS72XX_MODEL_TS7200		0x00
#define TS72XX_MODEL_TS7250		0x01
#define TS72XX_MODEL_TS7260		0x02
#define TS72XX_MODEL_TS7300             0x03
#define TS72XX_MODEL_TS7400             0x04


#define TS72XX_OPTIONS_PHYS_BASE	0x22400000
#define TS72XX_OPTIONS_VIRT_BASE	0xfebfe000
#define TS72XX_OPTIONS_SIZE		0x00001000

#define TS72XX_OPTIONS_COM2_RS485	0x02
#define TS72XX_OPTIONS_MAX197		0x01


#define TS72XX_OPTIONS2_PHYS_BASE	0x22800000
#define TS72XX_OPTIONS2_VIRT_BASE	0xfebfd000
#define TS72XX_OPTIONS2_SIZE		0x00001000

#define TS72XX_OPTIONS2_TS9420		0x04
#define TS72XX_OPTIONS2_TS9420_BOOT	0x02


#define TS72XX_NOR_PHYS_BASE		0x60000000
#define TS72XX_NOR2_PHYS_BASE		0x62000000

#define TS72XX_NAND1_DATA_PHYS_BASE	0x60000000
#define TS72XX_NAND2_DATA_PHYS_BASE	0x70000000
#define TS72XX_NAND_DATA_VIRT_BASE	0xfebfc000
#define TS72XX_NAND_DATA_SIZE		0x00001000

#define TS72XX_NAND1_CONTROL_PHYS_BASE	0x60400000
#define TS72XX_NAND2_CONTROL_PHYS_BASE	0x70400000
#define TS72XX_NAND_CONTROL_VIRT_BASE	0xfebfb000
#define TS72XX_NAND_CONTROL_SIZE	0x00001000

#define TS72XX_NAND1_BUSY_PHYS_BASE	0x60800000
#define TS72XX_NAND2_BUSY_PHYS_BASE	0x70800000
#define TS72XX_NAND_BUSY_VIRT_BASE	0xfebfa000
#define TS72XX_NAND_BUSY_SIZE		0x00001000

#define TS72XX_RTC_INDEX_VIRT_BASE	0xfebf9000
#define TS72XX_RTC_INDEX_PHYS_BASE	0x10800000
#define TS72XX_RTC_INDEX_SIZE		0x00001000

#define TS72XX_RTC_DATA_VIRT_BASE	0xfebf8000
#define TS72XX_RTC_DATA_PHYS_BASE	0x11700000
#define TS72XX_RTC_DATA_SIZE		0x00001000

#define TS7400_RTC_BASE                 0x80840000
#define TS7400_RTC_SIZE                 0x00001000

#define TS72XX_WATCHDOG_CONTROL_VIRT_BASE	0xfebf7000
#define TS72XX_WATCHDOG_CONTROL_PHYS_BASE	0x23800000
#define TS72XX_WATCHDOG_CONTROL_SIZE	0x00001000

#define TS72XX_WATCHDOG_FEED_VIRT_BASE	0xfebf6000
#define TS72XX_WATCHDOG_FEED_PHYS_BASE	0x23c00000
#define TS72XX_WATCHDOG_FEED_SIZE	0x00001000

#define TS72XX_PLD_VERSION_VIRT_BASE	0xfebf5000
#define TS72XX_PLD_VERSION_PHYS_BASE	0x23400000
#define TS72XX_PLD_VERSION_SIZE	0x00001000

#define TS72XX_RS485_CONTROL_VIRT_BASE	0xfebf4000
#define TS72XX_RS485_CONTROL_PHYS_BASE	0x22c00000
#define TS72XX_RS485_CONTROL_SIZE	0x00001000

#define TS72XX_RS485_MODE_VIRT_BASE	0xfebf3000
#define TS72XX_RS485_MODE_PHYS_BASE	0x23000000
#define TS72XX_RS485_MODE_SIZE	0x00001000

#define TS72XX_JUMPERS_MAX197_VIRT_BASE	0xfebf2000
#define TS72XX_JUMPERS_MAX197_PHYS_BASE	0x10800000
#define TS72XX_JUMPERS_MAX197_SIZE	0x00001000

#define TS72XX_MAX197_SAMPLE_VIRT_BASE	0xfebf1000
#define TS72XX_MAX197_SAMPLE_PHYS_BASE	0x10f00000
#define TS72XX_MAX197_SAMPLE_SIZE	0x00001000


#define TS72XX_PC104_8BIT_IO_VIRT_BASE  0xfebf0000
#define TS72XX_PC104_8BIT_IO_PHYS_BASE  0x11e00000
#define TS72XX_PC104_8BIT_IO_SIZE       0x00001000
#define TS72XX_PC104_8BIT_MEM_VIRT_BASE 0xfea00000
#define TS72XX_PC104_8BIT_MEM_PHYS_BASE 0x11a00000
#define TS72XX_PC104_8BIT_MEM_SIZE      0x00100000

#define TS72XX_PC104_16BIT_IO_VIRT_BASE  0xfebef000
#define TS72XX_PC104_16BIT_IO_PHYS_BASE  0x21e00000
#define TS72XX_PC104_16BIT_IO_SIZE       0x00001000
#define TS72XX_PC104_16BIT_MEM_VIRT_BASE 0xfe900000
#define TS72XX_PC104_16BIT_MEM_PHYS_BASE 0x21a00000
#define TS72XX_PC104_16BIT_MEM_SIZE      0x00100000

/*
 * TS7200 CF memory map:
 *
 * phys		size	description
 * 11000000	7	CF registers (8-bit each), starting at 11000001
 * 10400006	2	CF aux registers (8-bit)
 * 21000000	2	CF data register (16-bit)
 */

#define TS7200_CF_CMD_PHYS_BASE  0x11000000
#define TS7200_CF_AUX_PHYS_BASE  0x10400006
#define TS7200_CF_DATA_PHYS_BASE 0x21000000

#ifndef __ASSEMBLY__
#include <linux/io.h>

static inline int board_is_ts7200(void)
{
	return __raw_readb(TS72XX_MODEL_VIRT_BASE) == TS72XX_MODEL_TS7200;
}

static inline int board_is_ts7250(void)
{
	return __raw_readb(TS72XX_MODEL_VIRT_BASE) == TS72XX_MODEL_TS7250;
}

static inline int board_is_ts7260(void)
{
	return __raw_readb(TS72XX_MODEL_VIRT_BASE) == TS72XX_MODEL_TS7260;
}

static inline int board_is_ts7400(void)
{
        return (__raw_readb(TS72XX_MODEL_VIRT_BASE) & 0x7) == TS72XX_MODEL_TS7400;
}

static inline int is_max197_installed(void)
{
	return !!(__raw_readb(TS72XX_OPTIONS_VIRT_BASE) &
					TS72XX_OPTIONS_MAX197);
}

static inline int is_ts9420_installed(void)
{
	return !!(__raw_readb(TS72XX_OPTIONS2_VIRT_BASE) &
					TS72XX_OPTIONS2_TS9420);
}

static inline int is_rs485_installed(void)
{
	return !!(__raw_readb(TS72XX_OPTIONS_VIRT_BASE) &
					TS72XX_OPTIONS_COM2_RS485);
}

static inline int get_ts72xx_pld_version(void)
{
	return (__raw_readb(TS72XX_PLD_VERSION_VIRT_BASE) & 0x7);
}

static inline int is_jp6_set(void)
{
	return (__raw_readb(TS72XX_OPTIONS2_VIRT_BASE) & 0x1);
}

#endif
