/*
 *  arch/arm/mach-ep93xx/pm.c
 *
 *  EP93xx Power Management Routines
 *
 *  Based on pm.c from Andre Renaud, Bluewater Systems Ltd.
 *
 *  (c) Copyright 2008  Matthieu Crapet <mcrapet@gmail.com>
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version
 *  2 of the License, or (at your option) any later version.
 */

#include <linux/suspend.h>
#include <linux/sched.h>
#include <linux/proc_fs.h>
#include <linux/interrupt.h>
#include <linux/sysfs.h>
#include <linux/module.h>
#include <mach/hardware.h>


static inline void ep93xx_standby(void)
{
  u32 v;
  v = __raw_readl(EP93XX_SYSCON_DEVCFG);
  v |= EP93XX_SYSCON_DEVCFG_SHENA;
  __raw_writel(0xaa, EP93XX_SYSCON_SWLOCK);
  __raw_writel(v, EP93XX_SYSCON_DEVCFG);

  v = __raw_readl(EP93XX_SYSCON_STANDBY);

  asm("nop; nop; nop; nop; nop");
}

static inline void ep93xx_resume(void)
{
  u32 v;

  v = __raw_readl(EP93XX_SYSCON_DEVCFG);
  v &= ~EP93XX_SYSCON_DEVCFG_SHENA;
  __raw_writel(0xaa, EP93XX_SYSCON_SWLOCK);
  __raw_writel(v, EP93XX_SYSCON_DEVCFG);
}

static int suspend_ep93xx_enter(suspend_state_t state)
{
  switch (state) {
    case PM_SUSPEND_STANDBY:
    case PM_SUSPEND_MEM:
      ep93xx_standby(); /* go zzz */
      ep93xx_resume();
  }
  return 0;
}

static int suspend_ep93xx_valid(suspend_state_t state)
{
  return (state == PM_SUSPEND_STANDBY) ||
    (state == PM_SUSPEND_MEM);
}


static struct platform_suspend_ops ep93xx_suspend_ops = {
  .enter	= suspend_ep93xx_enter,
  .valid	= suspend_ep93xx_valid,
};

static int __init ep93xx_pm_init(void)
{
  pr_info("ep93xx: Power Management\n");
  suspend_set_ops(&ep93xx_suspend_ops);
  return 0;
}
__initcall(ep93xx_pm_init);
