/*
 * arch/arm/mach-ep93xx/ts72xx.c
 * Technologic Systems TS72xx SBC support.
 *
 * Copyright (C) 2006 Lennert Buytenhek <buytenh@wantstofly.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 */

#include <linux/kernel.h>
#include <linux/init.h>
#include <linux/mm.h>
#include <linux/sched.h>
#include <linux/interrupt.h>
#include <linux/ioport.h>
#include <linux/mtd/physmap.h>
#include <linux/platform_device.h>
#include <linux/m48t86.h>
#include <linux/io.h>
#include <linux/i2c.h>
#include <linux/spi/spi.h>
#include <mach/hardware.h>
#include <mach/spi.h>
#include <mach/gpio.h>
#include <asm/mach-types.h>
#include <asm/mach/arch.h>
#include <asm/mach/map.h>
#include <linux/delay.h>

static struct map_desc ts72xx_io_desc[] __initdata = {
	{
		.virtual	= TS72XX_MODEL_VIRT_BASE,
		.pfn		= __phys_to_pfn(TS72XX_MODEL_PHYS_BASE),
		.length		= TS72XX_MODEL_SIZE,
		.type		= MT_DEVICE,
  }, {
    .virtual  = TS72XX_PLD_VERSION_VIRT_BASE,
    .pfn    = __phys_to_pfn(TS72XX_PLD_VERSION_PHYS_BASE),
    .length   = TS72XX_PLD_VERSION_SIZE,
    .type   = MT_DEVICE,
	}, {
		.virtual	= TS72XX_OPTIONS_VIRT_BASE,
		.pfn		= __phys_to_pfn(TS72XX_OPTIONS_PHYS_BASE),
		.length		= TS72XX_OPTIONS_SIZE,
		.type		= MT_DEVICE,
	}, {
		.virtual	= TS72XX_OPTIONS2_VIRT_BASE,
		.pfn		= __phys_to_pfn(TS72XX_OPTIONS2_PHYS_BASE),
		.length		= TS72XX_OPTIONS2_SIZE,
		.type		= MT_DEVICE,
	}, {
		.virtual	= TS72XX_RTC_INDEX_VIRT_BASE,
		.pfn		= __phys_to_pfn(TS72XX_RTC_INDEX_PHYS_BASE),
		.length		= TS72XX_RTC_INDEX_SIZE,
		.type		= MT_DEVICE,
	}, {
		.virtual	= TS72XX_RTC_DATA_VIRT_BASE,
		.pfn		= __phys_to_pfn(TS72XX_RTC_DATA_PHYS_BASE),
		.length		= TS72XX_RTC_DATA_SIZE,
		.type		= MT_DEVICE,
	},
	/* Use this for debug only. Each device will map its own PC/104 address space */
	///* PC/104 (8-bit) I/O bus */
	//{
	//  .virtual  = TS72XX_PC104_8BIT_IO_VIRT_BASE,
	//  .pfn    = __phys_to_pfn(TS72XX_PC104_8BIT_IO_PHYS_BASE),
	//  .length   = TS72XX_PC104_8BIT_IO_SIZE,
	//  .type   = MT_DEVICE,
	//},
	///* PC/104 (16-bit) I/O bus */
	//{
	//  .virtual  = TS72XX_PC104_16BIT_IO_VIRT_BASE,
	//  .pfn    = __phys_to_pfn(TS72XX_PC104_16BIT_IO_PHYS_BASE),
	//  .length   = TS72XX_PC104_16BIT_IO_SIZE,
	//  .type   = MT_DEVICE,
	//},
	///* PC/104 (8-bit) MEM bus */
	//{
	//  .virtual  = TS72XX_PC104_8BIT_MEM_VIRT_BASE,
	//  .pfn    = __phys_to_pfn(TS72XX_PC104_8BIT_MEM_PHYS_BASE),
	//  .length   = TS72XX_PC104_8BIT_MEM_SIZE,
	//  .type   = MT_DEVICE,
	//},
	///* PC/104 (16-bit) MEM bus */
	//{
	//  .virtual  = TS72XX_PC104_16BIT_MEM_VIRT_BASE,
	//  .pfn    = __phys_to_pfn(TS72XX_PC104_16BIT_MEM_PHYS_BASE),
	//  .length   = TS72XX_PC104_16BIT_MEM_SIZE,
	//  .type   = MT_DEVICE,
	//}
};

static struct map_desc ts72xx_nand_io_desc[] __initdata = {
	{
		.virtual	= TS72XX_NAND_DATA_VIRT_BASE,
		.pfn		= __phys_to_pfn(TS72XX_NAND1_DATA_PHYS_BASE),
		.length		= TS72XX_NAND_DATA_SIZE,
		.type		= MT_DEVICE,
	}, {
		.virtual	= TS72XX_NAND_CONTROL_VIRT_BASE,
		.pfn		= __phys_to_pfn(TS72XX_NAND1_CONTROL_PHYS_BASE),
		.length		= TS72XX_NAND_CONTROL_SIZE,
		.type		= MT_DEVICE,
	}, {
		.virtual	= TS72XX_NAND_BUSY_VIRT_BASE,
		.pfn		= __phys_to_pfn(TS72XX_NAND1_BUSY_PHYS_BASE),
		.length		= TS72XX_NAND_BUSY_SIZE,
		.type		= MT_DEVICE,
	}
};

static struct map_desc ts72xx_alternate_nand_io_desc[] __initdata = {
	{
		.virtual	= TS72XX_NAND_DATA_VIRT_BASE,
		.pfn		= __phys_to_pfn(TS72XX_NAND2_DATA_PHYS_BASE),
		.length		= TS72XX_NAND_DATA_SIZE,
		.type		= MT_DEVICE,
	}, {
		.virtual	= TS72XX_NAND_CONTROL_VIRT_BASE,
		.pfn		= __phys_to_pfn(TS72XX_NAND2_CONTROL_PHYS_BASE),
		.length		= TS72XX_NAND_CONTROL_SIZE,
		.type		= MT_DEVICE,
	}, {
		.virtual	= TS72XX_NAND_BUSY_VIRT_BASE,
		.pfn		= __phys_to_pfn(TS72XX_NAND2_BUSY_PHYS_BASE),
		.length		= TS72XX_NAND_BUSY_SIZE,
		.type		= MT_DEVICE,
	}
};

static void __init ts72xx_map_io(void)
{
	ep93xx_map_io();
	iotable_init(ts72xx_io_desc, ARRAY_SIZE(ts72xx_io_desc));

	/*
	 * The TS-7200 has NOR flash, the other models have NAND flash.
	 */
	if (!board_is_ts7200()) {
		if (is_ts9420_installed()) {
			iotable_init(ts72xx_alternate_nand_io_desc,
				ARRAY_SIZE(ts72xx_alternate_nand_io_desc));
		} else {
			iotable_init(ts72xx_nand_io_desc,
				ARRAY_SIZE(ts72xx_nand_io_desc));
		}
	}
}

/* NOR flash (TS-7200 only) */

#if defined(CONFIG_MTD_PHYSMAP) || defined(CONFIG_MTD_PHYSMAP_MODULE)
static struct physmap_flash_data ts72xx_flash_data = {
	.width		= 2,
};

static struct resource ts72xx_flash_resource = {
	.start		= TS72XX_NOR_PHYS_BASE,
	.end		= TS72XX_NOR_PHYS_BASE + SZ_16M - 1, /* SZ_8M for 8mb flash */
	.flags		= IORESOURCE_MEM,
};

static struct platform_device ts72xx_flash = {
	.name		= "physmap-flash",
	.id		= 0,
	.dev		= {
		.platform_data	= &ts72xx_flash_data,
	},
	.num_resources	= 1,
	.resource	= &ts72xx_flash_resource,
};
#endif

/* Real time clock  */

static unsigned char ts72xx_rtc_readbyte(unsigned long addr)
{
	__raw_writeb(addr, TS72XX_RTC_INDEX_VIRT_BASE);
	return __raw_readb(TS72XX_RTC_DATA_VIRT_BASE);
}

static void ts72xx_rtc_writebyte(unsigned char value, unsigned long addr)
{
	__raw_writeb(addr, TS72XX_RTC_INDEX_VIRT_BASE);
	__raw_writeb(value, TS72XX_RTC_DATA_VIRT_BASE);
}

static struct m48t86_ops ts72xx_rtc_ops = {
	.readbyte	= ts72xx_rtc_readbyte,
	.writebyte	= ts72xx_rtc_writebyte,
};

static struct platform_device ts72xx_rtc_device = {
	.name		= "rtc-m48t86",
	.id		= -1,
	.dev		= {
		.platform_data	= &ts72xx_rtc_ops,
	},
	.num_resources	= 0,
};

static struct ep93xx_eth_data ts72xx_eth_data = {
	.phy_id		= 1,
};

/* Watchdog resource */

static struct resource ts72xx_watchdog_resources[] = {
  [0] = {
    .start  = TS72XX_WATCHDOG_CONTROL_PHYS_BASE,
    .end  = TS72XX_WATCHDOG_CONTROL_PHYS_BASE + 0x0fff,
    .flags  = IORESOURCE_MEM,
  },
  [1] = {
    .start  = TS72XX_WATCHDOG_FEED_PHYS_BASE,
    .end  = TS72XX_WATCHDOG_FEED_PHYS_BASE + 0x0fff,
    .flags  = IORESOURCE_MEM,
  },
};

static struct platform_device ts72xx_watchdog_device = {
	.name = "ts72xx-wdt",
	.id   = -1,
	.num_resources  = ARRAY_SIZE(ts72xx_watchdog_resources),
	.resource = ts72xx_watchdog_resources,
};

/* MAX197 (8 * 12-bit A/D converter) option */

static struct resource ts72xx_max197_resources[] = {
  [0] = { /* sample/control register */
    .start = TS72XX_MAX197_SAMPLE_PHYS_BASE,
    .end   = TS72XX_MAX197_SAMPLE_PHYS_BASE + TS72XX_MAX197_SAMPLE_SIZE - 1,
    .flags = IORESOURCE_MEM,
  },
  [1] = { /* busy bit */
    .start = TS72XX_JUMPERS_MAX197_PHYS_BASE,
    .end   = TS72XX_JUMPERS_MAX197_PHYS_BASE + TS72XX_JUMPERS_MAX197_SIZE - 1,
    .flags = IORESOURCE_MEM,
  }
};

static struct platform_device ts72xx_max197_device = {
	.name = "ts72xx-max197",
	.id   = -1,
	.dev  = {
		.platform_data  = NULL,
	},
	.num_resources  = ARRAY_SIZE(ts72xx_max197_resources),
	.resource = ts72xx_max197_resources,
};

/* SPI bus */

#if defined(CONFIG_SPI_TMP124) || defined(CONFIG_SPI_TMP124_MODULE)
void tmp124_spi_cs(u32 command) // FGPIO[2]
{
  if (command & SPI_CS_ASSERT) {
    gpio_set_value(EP93XX_GPIO_LINE_MCCD2, 0);
  } else if (command & SPI_CS_DEASSERT) {
    gpio_set_value(EP93XX_GPIO_LINE_MCCD2, 1);
  } else if (command & SPI_CS_INIT) {
    gpio_direction_output(EP93XX_GPIO_LINE_MCCD2, 1);
  }
}

static struct ep93xx_spi_chip tmp124_hw = {
	.cs_control = tmp124_spi_cs,
};

static struct spi_board_info ts72xx_spi_bus[] __initdata = {
	{
		/* TMP124 */
		.modalias      = "tmp124",
		.controller_data = &tmp124_hw,
		.bus_num       = 1,
		.chip_select   = 0,
		.max_speed_hz = 2 * 1000 * 1000,
	}
};
#endif

/***************************/
/*** TS-7400 RTC support ***/
/***************************/

static int config_ts7400_rtc = 0;
volatile unsigned int *dat_dr, *dat_ddr, *cs_dr, *cs_ddr, *ale_ddr, *ale_dr;
volatile unsigned int *rd_dr, *rd_ddr, *wr_dr, *wr_ddr;
#define DAT_DR          0x40000
#define DAT_DDR         0x40010
#define CS_DR           0x40040
#define CS_DDR          0x40044
#define ALE_DR          0x40040
#define ALE_DDR         0x40044
#define RD_DR           0x40040
#define RD_DDR          0x40044
#define WR_DR           0x40030
#define WR_DDR          0x40034

static void inline setRTCdata(unsigned char ch)
{
        *dat_dr = (unsigned int)ch;
        udelay(10);
        *dat_ddr = 0xFF;
}

static unsigned char inline getRTCdata(void)
{
        *dat_ddr = 0x00;
        udelay(10);
        return (unsigned char)(*dat_dr & 0xFF);
}

static void inline setRTC_RD(int en)
{
        if (!en)
                *rd_dr |= (1 << 3);
        else
                *rd_dr &= ~(1 << 3);
        *rd_ddr |= (1 << 3);
}

static void inline setRTC_CS(int en)
{
        if (!en)
                *cs_dr |= (1 << 5);
        else
                *cs_dr &= ~(1 << 5);
        *cs_ddr |= (1 << 5);
        udelay(10);
}

static void inline setRTC_ALE(int en)
{
        if (en)
                *ale_dr |= (1 << 4);
        else
                *ale_dr &= ~(1 << 4);
        *ale_ddr |= (1 << 4);
        udelay(10);
}

static void inline setRTC_WR(int en)
{
        if (!en)
                *wr_dr |= (1 << 3);
        else
                *wr_dr &= ~(1 << 3);
        *wr_ddr |= (1 << 3);
        udelay(10);
}

static void ts7400_rtc_writebyte(unsigned char value, unsigned long addr)
{
        if (!config_ts7400_rtc) {
                ale_ddr = (unsigned int *)(EP93XX_APB_VIRT_BASE + ALE_DDR);
                ale_dr = (unsigned int *)(EP93XX_APB_VIRT_BASE + ALE_DR);
                cs_ddr = (unsigned int *)(EP93XX_APB_VIRT_BASE + CS_DDR);
                cs_dr = (unsigned int *)(EP93XX_APB_VIRT_BASE + CS_DR);
                wr_ddr = (unsigned int *)(EP93XX_APB_VIRT_BASE + WR_DDR);
                wr_dr = (unsigned int *)(EP93XX_APB_VIRT_BASE + WR_DR);
                rd_ddr = (unsigned int *)(EP93XX_APB_VIRT_BASE + RD_DDR);
                rd_dr = (unsigned int *)(EP93XX_APB_VIRT_BASE + RD_DR);
                dat_ddr = (unsigned int *)(EP93XX_APB_VIRT_BASE + DAT_DDR);
                dat_dr = (unsigned int *)(EP93XX_APB_VIRT_BASE + DAT_DR);

                __raw_writel(0xaa, EP93XX_SYSCON_SWLOCK);
                __raw_writel(0x08140d00, EP93XX_SYSCON_DEVCFG);

                config_ts7400_rtc = 1;
                //ensure RTC battery is enabled
                ts7400_rtc_writebyte(0x20, 0xa);        //enable the battery
        }
        setRTC_ALE(0);
        setRTC_RD(0);
        setRTC_CS(0);
        setRTC_WR(0);
        setRTC_RD(0);
        setRTC_CS(1);
        setRTC_ALE(1);
        setRTCdata(addr);
        setRTC_ALE(0);
        setRTC_WR(1);
        setRTCdata(value);
        setRTC_WR(0);
        setRTC_CS(0);
}

static unsigned char ts7400_rtc_readbyte(unsigned long addr)
{
        unsigned char dat;
        if (!config_ts7400_rtc) {
                ale_ddr = (unsigned int *)(EP93XX_APB_VIRT_BASE + ALE_DDR);
                ale_dr = (unsigned int *)(EP93XX_APB_VIRT_BASE + ALE_DR);
                cs_ddr = (unsigned int *)(EP93XX_APB_VIRT_BASE + CS_DDR);
                cs_dr = (unsigned int *)(EP93XX_APB_VIRT_BASE + CS_DR);
                wr_ddr = (unsigned int *)(EP93XX_APB_VIRT_BASE + WR_DDR);
                wr_dr = (unsigned int *)(EP93XX_APB_VIRT_BASE + WR_DR);
                rd_ddr = (unsigned int *)(EP93XX_APB_VIRT_BASE + RD_DDR);
                rd_dr = (unsigned int *)(EP93XX_APB_VIRT_BASE + RD_DR);
                dat_ddr = (unsigned int *)(EP93XX_APB_VIRT_BASE + DAT_DDR);
                dat_dr = (unsigned int *)(EP93XX_APB_VIRT_BASE + DAT_DR);

                __raw_writel(0xaa, EP93XX_SYSCON_SWLOCK);
                __raw_writel(0x08140d00, EP93XX_SYSCON_DEVCFG);

                config_ts7400_rtc = 1;
                //ensure RTC battery is enabled
                ts7400_rtc_writebyte(0x20, 0xa);        //enable the battery
        }
        setRTC_ALE(0);
        setRTC_RD(0);
        setRTC_CS(0);
        setRTC_WR(0);
        setRTC_RD(0);
        setRTC_CS(1);
        setRTC_ALE(1);
        setRTCdata(addr);
        setRTC_ALE(0);
        getRTCdata();
        setRTC_RD(1);
        dat = getRTCdata();
        setRTC_RD(0);
        setRTC_CS(0);
        return dat;
}

static struct m48t86_ops ts7400_rtc_ops = {

        .readbyte = ts7400_rtc_readbyte,
        .writebyte = ts7400_rtc_writebyte,
};

static struct platform_device ts7400_rtc_device = {
        .name = "rtc-m48t86",
        .id = -1,
        .dev = {
                .platform_data = &ts7400_rtc_ops,
                },
        .num_resources = 0,
};


static void __init ts72xx_init_machine(void)
{
	ep93xx_init_devices();

#if defined(CONFIG_MTD_PHYSMAP) || defined(CONFIG_MTD_PHYSMAP_MODULE)
	if (board_is_ts7200())
		platform_device_register(&ts72xx_flash);
#endif

        if (board_is_ts7400()) 
          platform_device_register(&ts7400_rtc_device);
        else 
	  platform_device_register(&ts72xx_rtc_device);

	platform_device_register(&ts72xx_watchdog_device);

	if (is_max197_installed()) {
		platform_device_register(&ts72xx_max197_device);
	}

	#if defined(CONFIG_SPI_TMP124) || defined(CONFIG_SPI_TMP124_MODULE)
	spi_register_board_info(ts72xx_spi_bus, ARRAY_SIZE(ts72xx_spi_bus));
	#endif

	ep93xx_register_eth(&ts72xx_eth_data, 1);
}

MACHINE_START(TS72XX, "Technologic Systems TS-72xx SBC")
	/* Maintainer: Lennert Buytenhek <buytenh@wantstofly.org> */
	.phys_io	= EP93XX_APB_PHYS_BASE,
	.io_pg_offst	= ((EP93XX_APB_VIRT_BASE) >> 18) & 0xfffc,
	.boot_params	= EP93XX_SDCE3_PHYS_BASE_SYNC + 0x100,
	.map_io		= ts72xx_map_io,
	.init_irq	= ep93xx_init_irq,
	.timer		= &ep93xx_timer,
	.init_machine	= ts72xx_init_machine,
MACHINE_END
