/*
 *  Technologic Systems TS-7200 Compact Flash IDE device driver.
 *
 * (c) Copyright 2009  Matthieu Crapet <mcrapet@gmail.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 */

#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/ide.h>

#include <asm/io.h>
#include <asm/irq.h>

static __init int ide_ts7200_cf_init(void)
{
  hw_regs_t hw, *hws[] = { &hw, NULL, NULL, NULL };
  void __iomem *base, *ctl, *data;

  base = ioremap(TS7200_CF_CMD_PHYS_BASE, 0x10);  // 8-bit access
  ctl  = ioremap(TS7200_CF_AUX_PHYS_BASE, 0x10);  // 8-bit access (usually base+0x206)
  data = ioremap(TS7200_CF_DATA_PHYS_BASE, 0x10); // 16-bit access

  if ((base != NULL) && (ctl != NULL) && (data != NULL)) {
    memset(&hw, 0, sizeof(hw));

    ide_std_init_ports(&hw, (unsigned long)base, (unsigned long)ctl);
    hw.io_ports.data_addr = (unsigned long)data;

    hw.irq = IRQ_EP93XX_EXT0;
    hw.chipset = ide_generic;

    return ide_host_add(NULL, hws, NULL);
  }

  if (base) iounmap(base);
  if (ctl)  iounmap(ctl);
  if (data) iounmap(data);

  return -ENODEV;
}


module_init(ide_ts7200_cf_init);

MODULE_AUTHOR("Matthieu Crapet <mcrapet@gmail.com>");
MODULE_DESCRIPTION("TS-7200 Compact Flash IDE driver");
MODULE_LICENSE("GPL");
MODULE_VERSION("0.1");
