/*
 *  TS-72xx keypad device driver for DIO1 header (DIO_0 thru DIO_7 are using port B)
 *
 * (c) Copyright 2008  Matthieu Crapet <mcrapet@gmail.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 */

#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/init.h>
#include <linux/platform_device.h>
#include <linux/input.h>

#include <mach/ep93xx-keypad.h>

/* Port B = C0 R3 C1 R2 C2 C3 R1 R0
 * (i.e. row0 is bit 0, row1 is bit 1, ...)
 */
static struct ep93xx_gpio_portx_keypad_platform_data kp_portb_4x4 = {
  .nr_rows = 4,
  .nr_cols = 4,
  { { KEY_7, KEY_8, KEY_9, KEY_F },
    { KEY_4, KEY_5, KEY_6, KEY_E },
    { KEY_1, KEY_2, KEY_3, KEY_D },
    { KEY_A, KEY_0, KEY_B, KEY_C }
  },
  .gpio_rows = { 0, 1, 4, 6 },
  .gpio_cols = { 7, 5, 3, 2 },
};


static void ts72xx_dio_release(struct device *dev)
{
  // nothing to do (no kfree) because we have static struct
}

static struct platform_device kp_portb_4x4_device = {
  .name = "ep93xx-gpio-keypad",
  .id   = -1, // one instance only
  .dev    = {
    .platform_data = &kp_portb_4x4,
    .release = ts72xx_dio_release,
  },
};

static int __init ts72xx_dio_init(void)
{
  return platform_device_register(&kp_portb_4x4_device);
}

static void __exit ts72xx_dio_exit(void)
{
  platform_device_unregister(&kp_portb_4x4_device);
}

module_init(ts72xx_dio_init);
module_exit(ts72xx_dio_exit);

MODULE_AUTHOR("Matthieu Crapet <mcrapet@gmail.com>");
MODULE_DESCRIPTION("Platform device 4x4 keypad");
MODULE_LICENSE("GPL");
