
#include <linux/module.h>
#include <linux/moduleparam.h>
#include <linux/init.h>

#include <linux/sched.h>
#include <linux/kernel.h>	/* printk() */
#include <linux/slab.h>		/* kmalloc() */
#include <linux/fs.h>		/* everything... */
#include <linux/errno.h>	/* error codes */
#include <linux/timer.h>
#include <linux/types.h>	/* size_t */
#include <linux/fcntl.h>	/* O_ACCMODE */
#include <linux/hdreg.h>	/* HDIO_GETGEO */
#include <linux/kdev_t.h>
#include <linux/vmalloc.h>
#include <linux/genhd.h>
#include <linux/blkdev.h>
#include <linux/buffer_head.h>	/* invalidate_bdev */
#include <linux/bio.h>

#include <linux/irq.h>
#include <linux/interrupt.h>

#include <linux/slab.h>
#include <linux/mm.h>
#include <asm/io.h>
#include <asm/cacheflush.h>

#include <linux/suspend.h>
#include <linux/kthread.h>
#include <asm/atomic.h>

extern void __udelay(unsigned int);
#define udelay(x) __udelay((x))

/* The kernel talks to us in terms of small sectors, always. */
#define KERNEL_SECTOR_SIZE	512

#define SD_DEFNAME		"tssdcard"
#define SD_DEFIOSIZE		4
#define SD_BLKSIZE		1024	//nsectors?
#define SD_HARDSECT		512
#define SD_SHIFT		4	//max 16 partitions 2^4
#define SD_NUMPART		1 << 4

#define TSSDCARD_MINORS		SD_NUMPART
#define MINOR_SHIFT		SD_SHIFT
#define DEVNUM(kdevnum)		(MINOR(kdev_t_to_nr(kdevnum)) >> MINOR_SHIFT

#define BIGENDIAN
#include 			"tssdcore2.c"

#define DMA_ENABLE 		0
#define THREAD_ENABLE 		0

#define DMA_CHANNEL		0

#define TS7800_IRQ_DMA		65
#define TS7800_IRQ_WAIT		66

#define TS7XXX_SDCARD1		0xE8000100
#define TS7800_DMAADDR		0xE8000400

#define TS7800_FPGASDDAT	0x80000104
#define TS7800_CPUDOORBELL	0xF1020400
#define TS7800_FPGADOORBELL	0xE8000200
#define TS7800_FPGAMSIMASK	0xE8000204
#define TS7800_FPGAIRQRAW	0xE8000208

#define DOORBELL_DMABIT		0x1 << 1
#define DOORBELL_WAITBIT	0x1 << 2

#define NDEVICES 1 

enum {
	RM_SIMPLE = 0,		/* The extra-simple request function */
	RM_FULL = 1,		/* The full-blown version */
	RM_NOQUEUE = 2,		/* Use make_request */
};

static unsigned char *name = SD_DEFNAME;
static unsigned int io = TS7XXX_SDCARD1;
static int dmaenable = DMA_ENABLE;
static int threnable = THREAD_ENABLE;
static int dmach = DMA_CHANNEL;
static int reqmode = RM_FULL;
static int tssdcard_major = 0;
static int nsectors = SD_BLKSIZE;
static int ndevices = NDEVICES;
static struct semaphore sem;
static atomic_t busy;

MODULE_AUTHOR("Technologic Systems");
MODULE_LICENSE("GPL");

struct tssdcard_dev {
	struct sdcore tssdcore;

	char *devname;
	int size;		/* Device size in sectors */

	struct gendisk *gd;	/* The gendisk structure */

	struct timer_list timer;

	short media_change;	/* Flag a media change? */

	int irq, irq_wait;
	unsigned long dmaregs;
	int dmach;

	atomic_t users;		/* How many users */
	atomic_t busy;
	int parksect;

	struct request_queue *queue;	/* The device request queue */
	struct bio *bio;
	struct bio *biotail;

	struct task_struct *thread;

	wait_queue_head_t event;
	wait_queue_head_t qdma;
	wait_queue_head_t qwait;

	spinlock_t lock;

};

static struct tssdcard_dev *devices = NULL;
static struct block_device_operations tssdcard_ops;

#define DMA_DECLBASE(x)	unsigned long dmaregs = (x);

static irqreturn_t int_handler(int irq, void *vidp, struct pt_regs regs)
{
	struct tssdcard_dev *dev = (struct tssdcard_dev *)vidp;

	//BUG_ON(irq != dev->irq || irq != dev->irq_wait);
	//printk("irq %d\n",irq);

	if (irq == dev->irq)
		wake_up_interruptible(&dev->qdma);
	else if (irq == dev->irq_wait)
		wake_up_interruptible(&dev->qwait);

	disable_irq_nosync(irq);

	return IRQ_HANDLED;
}

static void tssdcard_irqwait(void *os_arg, int debug)
{
	printk("tssdcard_irqwait()\n");
	printk("implement me for the amcc 440EP\n");
	return;
}



static void tssdcard_delay(void *arg, unsigned int us)
{
	udelay(us);
}

/*
 * Handle an I/O request.
 */
static inline int tssdcard_transfer(struct tssdcard_dev *dev, unsigned long
		sector, unsigned long nsect, char *buffer, int rw) { 
	int ret;
int i;
	///? unlock here for interrupt handling spin_unlock_irq(&dev->lock);

//printk("size:%d sector:%d amount:%d rw:%d\n", dev->size, sector, nsect, rw);


	if ((sector + nsect) > (dev->size * 512 / KERNEL_SECTOR_SIZE)) {
		printk(KERN_NOTICE "Beyond-end write (%ld %ld)\n", sector,
		       nsect);
		printk("size of disk = 0x%lx\n", (dev->size * 512 / KERNEL_SECTOR_SIZE));
		return;	//EWD: debugging
	}


	del_timer(&dev->timer);

	switch (rw) {
	case WRITE:
		ret = sdwrite(&dev->tssdcore, sector, buffer, nsect);
		if (ret && !dev->tssdcore.sd_wprot) {
			sdreset(&dev->tssdcore);
			ret = sdwrite(&dev->tssdcore, sector, buffer, nsect);
		}
		//if (ret == 0) mark_buffer_uptodate(buffer, 1);
		dev->timer.expires = jiffies + (HZ / 2);
		add_timer(&dev->timer);
		break;

	case READ:
	case READA:

		ret = sdread(&dev->tssdcore, sector, buffer, nsect);

		if (ret) {
			sdreset(&dev->tssdcore);
			ret = sdread(&dev->tssdcore, sector, buffer, nsect);
		}

		dev->parksect = sector + nsect - 1;

		//sdread will park at sector+nsect+1, if we read the last sector
		//this means we are parked on an invalid sector so reset the card.
		if ((sector + nsect) >= (dev->size * 512 / KERNEL_SECTOR_SIZE))
			sdreset(&dev->tssdcore);

		break;
	}

	///? lock back here for interrupt handling
	//spin_lock(&dev->lock);  
	
	return ret;
}

/*
 * Smarter request function that "handles clustering".
 */
static inline void tssdcard_handle_fullreq(struct tssdcard_dev *dev)
{
	struct request *req;
	struct bio *bio;
	struct bio_vec *bvec;
	int sectors_xferred;
	struct req_iterator iter;
	sector_t sector, end_sector, n_sectors;
	char *buffer;
	int error;
printk("tssdcard_handle_fullreq()\n");
	//Transfer a set of clustered requests
	while ((req = elv_next_request(dev->queue)) != NULL) {

		if (!blk_fs_request(req)) {
			printk(KERN_NOTICE "Skip non-fs request\n");
			end_request(req, 0);
			continue;
		}


		//Transfer a full request.
		sectors_xferred = 0;
		rq_for_each_segment(bvec, req, iter) {
			unsigned long flags;
			sector = iter.bio->bi_sector;
			end_sector = sector + (iter.bio->bi_size >> 9) - 1;

			if((sector + (bvec->bv_len >> 9)) > end_sector)
				n_sectors = end_sector - sector + 1;
			else
				n_sectors = bvec->bv_len >> 9;
			if(n_sectors == 0) continue;


			buffer = bvec_kmap_irq(bvec, &flags);

			error = tssdcard_transfer(dev, sector, n_sectors, buffer,
					  bio_data_dir(iter.bio));

			sector += n_sectors;
			flush_kernel_dcache_page(bvec->bv_page);
			bvec_kunmap_irq(bvec, &flags);
			sectors_xferred += iter.bio->bi_size / KERNEL_SECTOR_SIZE;
		}
//EWD: why are we hardcoding there were no errors? is 1 =>
		error=0;
		__blk_end_request(req, error, sectors_xferred << 9);

	}

}
static void tssdcard_full_request(struct request_queue *q)
{
	struct tssdcard_dev *dev = q->queuedata;

	atomic_inc(&dev->busy);
	if (atomic_read(&dev->busy) > 1)
		panic("recursive make_request!\n");

	if (threnable)
		wake_up(&dev->event);
	else
		tssdcard_handle_fullreq(dev);

	atomic_dec(&dev->busy);

	return;
}

static inline void tssdcard_add_bio(struct tssdcard_dev *dev, struct bio *bio)
{
	if (dev->biotail) {
		dev->biotail->bi_next = bio;
		dev->biotail = bio;
	} else
		dev->bio = dev->biotail = bio;
}

static inline struct bio *tssdcard_get_bio(struct tssdcard_dev *dev)
{
	struct bio *bio;

	if ((bio = dev->bio)) {
		if (bio == dev->biotail)
			dev->biotail = NULL;
		dev->bio = bio->bi_next;
		bio->bi_next = NULL;
	}

	return bio;
}


/*
 * Open and close.
 */
static int tssdcard_open(struct block_device *bdev, fmode_t mode)
{

	struct tssdcard_dev *dev = bdev->bd_disk->private_data;
	int ret = 0;

	if (!atomic_read(&dev->users)) {
		check_disk_change(bdev);
	}

	atomic_inc(&dev->users);

	return 0;
}

static int tssdcard_release(struct gendisk *disk, fmode_t mode)
{
	struct tssdcard_dev *dev = disk->private_data;

	//if (debug) printk("release\n");

	atomic_dec(&dev->users);
	if (atomic_read(&dev->users) == 0) {
		if (dev->thread != NULL) {
			kthread_stop(dev->thread);
			dev->thread = NULL;
		}
	}

	return 0;
}

int tssdcard_media_changed(struct gendisk *gd)
{
	struct tssdcard_dev *dev = gd->private_data;
	char buf[512];
	return sdread(&dev->tssdcore, 1, buf, 1); 
}

int tssdcard_revalidate(struct gendisk *gd)
{
	struct tssdcard_dev *dev = gd->private_data;

	dev->size = sdreset(&dev->tssdcore);
	if (dev->size)
		set_capacity(dev->gd, dev->size * 512 / KERNEL_SECTOR_SIZE);

	return 0;
}

static void tssdcard_sdcommit(unsigned long ldev)
{
	struct tssdcard_dev *dev = (struct tssdcard_dev *)ldev;
	char buf[512];
	sdread(&dev->tssdcore, dev->parksect, buf, 1);
}

static int tssdcard_shutdownhook(struct notifier_block *NotifierBlock,
				 unsigned long Event, void *Buffer)
{
	int i;
	///? del_timer(&devices->timer);
	if (!
	    (Event == SYS_RESTART || Event == SYS_HALT
	     || Event == SYS_POWER_OFF))
		return NOTIFY_DONE;

	for (i = 0; i < ndevices; i++) {
		tssdcard_sdcommit((unsigned long)(devices + i));
	}

	return NOTIFY_OK;
}

static struct notifier_block shutdownhook_notifier =
    { tssdcard_shutdownhook, NULL, 0 };

/*
 * The ioctl() implementation
 */
int tssdcard_ioctl(struct block_device *bdev, fmode_t mode,
	                    unsigned int cmd, unsigned long arg)
{
	struct hd_geometry geo;
	struct tssdcard_dev *dev = bdev->bd_disk->private_data;

	switch (cmd) {
	case HDIO_GETGEO:
		/*
		 * Get geometry: since we are a virtual device, we have to make
		 * up something plausible.  So we claim 16 sectors, four heads,
		 * and calculate the corresponding number of cylinders.  We set the
		 * start of data at sector four.
		 */
		geo.cylinders = dev->size / (16 * 32);
		geo.heads = 16;
		geo.sectors = 32;
		geo.start = 0;
		if (copy_to_user((void __user *)arg, &geo, sizeof(geo)))
			return -EFAULT;
		return 0;
	}

	return -ENOTTY;		/* unknown command */
}

/*
 * The device operations structure.
 */
static struct block_device_operations tssdcard_ops = {
	.owner = THIS_MODULE,
	.open = tssdcard_open,
	.release = tssdcard_release,
	.media_changed = tssdcard_media_changed,
	//.revalidate_disk = tssdcard_revalidate,	//EWD: additional calls
	//to sdreset return size of 0?
	.ioctl = tssdcard_ioctl
};

static inline void tssdcard_handle_simplereq(struct tssdcard_dev *dev)
{
	struct request *req;
	int error;
	while ((req = elv_next_request(dev->queue)) != NULL) {
		unsigned block = req->sector;
		unsigned count = req->nr_sectors;
		int rw = rq_data_dir(req);
		if (! blk_fs_request(req)) {
			printk (KERN_NOTICE "Skip non-fs request\n");
			end_request(req, 0);
			continue;
		}
		if (block + count > get_capacity(req->rq_disk)) {
			end_request(req, 0);
			continue;
		}
		if (rw != READ && rw != WRITE) {
			printk("tssdcard_handle_simplereq: unknown request\n");
			end_request(req, 0);
			continue;
		}



		//error = tssdcard_transfer(dev, block, count, req->buffer, rw);
		tssdcard_transfer(dev, req->sector, req->current_nr_sectors, req->buffer, rq_data_dir(req));
//EWD: don't hardcode no error!!!
		end_request(req, 1);  /* wrap up, 0 = fail, 1 = success */
	}

}

static inline void handle_request (struct request_queue *q, struct bio *bio)
{
	struct tssdcard_dev *dev = q->queuedata;
	//spin_unlock_irq(&dev->lock);

	if (down_interruptible(&sem))
		return -ERESTARTSYS;
	atomic_inc(&busy);
	if (atomic_read(&busy) > 1)
		panic("recursive make_request!\n");

	tssdcard_handle_simplereq(dev);

	atomic_dec(&busy);
	up(&sem);
}



/* Request Mode RM_SIMPLE=0 */
static void tssdcard_request(struct request_queue *q)
{
	return handle_request(q, NULL);
}

/*
 * Set up our internal device.
 */
static void setup_device(struct tssdcard_dev *dev, int which)
{
	static unsigned int adr = 0;
	unsigned int scratchreg0;
	unsigned long conreg;

	printk("%s: Technologic Systems SD card controller, address 0x%x\n",
	       name, io);

	memset(dev, 0, sizeof(struct tssdcard_dev));

	//IO Remapping(use the same virtual address for all LUNs)
	if (!adr)
		adr = (unsigned long)ioremap(0x13000000, SD_DEFIOSIZE);
	dev->tssdcore.sd_regstart = adr;
	dev->tssdcore.sd_lun = which;

	//verify previous init
	conreg = (unsigned long)0;
	scratchreg0 = 0;
	if (scratchreg0) {
		dev->tssdcore.sdboot_token = scratchreg0;
		outl(0, 0);
	}

	dev->tssdcore.os_irqwait = NULL;
	dev->tssdcore.os_arg = dev;
	dev->tssdcore.os_delay = tssdcard_delay;
	dev->tssdcore.os_dmastream = NULL;
	dev->tssdcore.os_dmaprep = NULL;
	dev->tssdcore.sd_writeparking = 1;
	dev->irq = 0;
	dev->dmach = 0;
	dev->dmaregs = 0;
	dev->parksect = 0;

	sema_init(&sem, 1);
	atomic_set(&dev->users, 0);
	atomic_set(&dev->busy, 0);

	//SD Card size and Reset
	dev->size = sdreset(&dev->tssdcore);
	printk("size=0x%x\n", dev->size);
	dev->size = sdreset(&dev->tssdcore);
	printk("size=0x%x\n", dev->size);
	if (dev->size == 0) {
		printk("%s: no card found.\n", name);
		return;
	}

	dev->gd = alloc_disk(TSSDCARD_MINORS);
	if (!dev->gd) {
		printk(KERN_NOTICE "alloc_disk failure\n");
		return;
	}
	dev->devname = kmalloc(32, GFP_KERNEL);
	strcpy(dev->devname, name);
	snprintf(dev->gd->disk_name, 32, strcat(dev->devname, "%c"),
		 which + 'a');
	strcpy(dev->devname, dev->gd->disk_name);
	printk("%s: card /dev/%s has %d sectors (LUN %d)\n", name, dev->devname,
	       dev->size, which);

	spin_lock_init(&dev->lock);
	//The timer which "invalidates" the device.
	init_timer(&dev->timer);
	dev->timer.data = (unsigned long)dev;
	dev->timer.function = tssdcard_sdcommit;
	dev->bio = dev->biotail = NULL;
	//dev->queue = blk_init_queue(tssdcard_full_request, &dev->lock);
dev->queue = blk_init_queue(tssdcard_request, &dev->lock);
blk_queue_max_sectors(dev->queue, 255);	//EWD: we didn't specific this previously
	if (dev->queue == NULL)
		return;

	blk_queue_hardsect_size(dev->queue, 512);
	dev->queue->queuedata = dev;

	//And the gendisk structure.
	dev->gd->major = tssdcard_major;
	dev->gd->first_minor = which * TSSDCARD_MINORS;
	dev->gd->flags = GENHD_FL_REMOVABLE;
	dev->gd->fops = &tssdcard_ops;
	dev->gd->queue = dev->queue;
	dev->gd->private_data = dev;

	set_capacity(dev->gd, dev->size * 512 / KERNEL_SECTOR_SIZE);
	add_disk(dev->gd);

	//udelay(1000); 

	return;
}

static int __init tssdcard_init(void)
{
	int i;

	tssdcard_major = register_blkdev(tssdcard_major, name);
	if (tssdcard_major <= 0) {
		printk(KERN_WARNING "tssdcard: unable to get major number\n");
		return -EBUSY;
	}

	devices = kmalloc(ndevices * sizeof(struct tssdcard_dev), GFP_KERNEL);
	if (devices == NULL)
		goto out_unregister;

	for (i = 0; i < ndevices; i++)
		setup_device(devices + i, i);

	return 0;

      out_unregister:
	unregister_blkdev(tssdcard_major, name);
	return -ENOMEM;
}

static void tssdcard_exit(void)
{
	struct tssdcard_dev *dev;
	int i;

	for (i = 0; i < ndevices; i++) {
		dev = devices + i;

		if (dev == NULL)
			continue;

		del_timer_sync(&dev->timer);

		if (dev->gd) {
			del_gendisk(dev->gd);
			put_disk(dev->gd);
		}

		if (dev->thread != NULL) {
			kthread_stop(dev->thread);
			dev->thread = NULL;
		}

		if (dmaenable) {
			free_irq(dev->irq, dev);
			free_irq(dev->irq_wait, dev);
		}

		iounmap((unsigned long *)dev->tssdcore.sd_regstart);
		iounmap((unsigned long *)dev->dmaregs);

	}

	unregister_blkdev(tssdcard_major, name);
	kfree(devices);
}

module_init(tssdcard_init);
module_exit(tssdcard_exit);
