/*
 *  EP93xx SPI (simple) include
 *
 * (c) Copyright 2008  Matthieu Crapet <mcrapet@gmail.com>
 * Based on pxa2xx_spi.c by Stephen Street / StreetFire Sound Labs
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 */

/* SSP Registers */
#define SSPCR0   0x00 /* Control register 0 */
#define SSPCR1   0x04 /* Control register 1 */
#define SSPDR    0x08 /* Receice FIFO data register (16-bit read) */
                      /* Transmit FIFO data register (16-bit write) */
#define SSPSR    0x0C /* Status register */
#define SSPCPSR  0x10 /* Clock prescale register (from 2 to 254, even number) */
#define SSPIxR   0x14 /* Interrupt identification register (read) */
                      /* Interrupt clear register (write) */

/* SSP control registers bit fields & masks */
#define SSP_CONTROL_SCR(x)         (((x) & 0xFF) << 8) /* Serial clock rate = SCLKOUT / CPSDVR / (1+SCR) */
#define SSP_CONTROL_SPH            (1 << 7)            /* SCLKOUT phase (for SPI only) */
#define SSP_CONTROL_SPO            (1 << 6)            /* SCLKOUT polarity (for SPI only) */
#define SSP_CONTROL_FRF(x)         (((x) & 3) << 4)    /* Frame format (0=SPI) */
#define SSP_CONTROL_DSS_4BIT_DATA   3
#define SSP_CONTROL_DSS_8BIT_DATA   7
#define SSP_CONTROL_DSS_15BIT_DATA 14
#define SSP_CONTROL_DSS_16BIT_DATA 15
#define SSP_CONTROL_DSS_MASK        0xF
#define SSP_CONTROL_MS              (1 << 5)           /* 0=master, 1=slave (can be modified when SSE=0) */
#define SSP_CONTROL_SSE             (1 << 4)           /* SSP operation enable (=1), disable (=0) */
#define SSP_CONTROL_LBM             (1 << 3)           /* Loop back mode */
#define SSP_CONTROL_RORIE           (1 << 2)           /* Interrupt enable : overrun condition */
#define SSP_CONTROL_TIE             (1 << 1)           /* Interrupt enable : transmit fifo */
#define SSP_CONTROL_RIE             (1 << 0)           /* Interrupt enable : receive fifo */

/* SSP status register (read only) */
#define SSP_STATUS_BUSY             (1 << 4)           /* Busy flag (0: SSP is idle) */
#define SSP_STATUS_RFF              (1 << 3)           /* Receive fifo full ? (1=full) */
#define SSP_STATUS_RNE              (1 << 2)           /* Receive fifo not empty ? (0=empty) */
#define SSP_STATUS_TNF              (1 << 1)           /* Transmit fifo not full ? (0=full) */
#define SSP_STATUS_TFE              (1 << 0)           /* Transmit fifo empty ? (1=empty) */

/* SSP SSPIIR/SSPICR register (write 1 to clear interrupt) */
#define SSP_SSPIxx_RORIS            (1 << 2)           /* Receive fifo overrun interrupt status */
#define SSP_SSPIxx_TIS              (1 << 1)           /* Transmit fifo service request interrupt status */
#define SSP_SSPIxx_RIS              (1 << 0)           /* Receive fifo service request interrupt status */

/* Default configuration values */
#define SPI_DEFAULT0 (SSP_CONTROL_DSS_16BIT_DATA | SSP_CONTROL_FRF(0) | SSP_CONTROL_SCR(0))
#define SPI_DEFAULT1 (SSP_CONTROL_SSE | SSP_CONTROL_RORIE)
#define SPI_DEFAULT_DIVISOR 254
